#!/bin/bash
# Based on the work of Terry Gray University of Washington
# This version by Robert Shingledecker

. /etc/init.d/dsl-functions

if [ $UID != 0 ]; then
  echo "Got root?"
  echo -n "Press Enter key. "; read gagme
  exit 1
fi    

MIRROR=$(dslrc.lua Mirror) || MIRROR="ibiblio.org/pub/Linux/distributions/damnsmall/"
PROTOCOL=$(dslrc.lua Protocol) || PROTOCOL="http"

echo
echo "${RED}DSL ${WHITE}Compressed Image ${BLUE}Install/Upgrade.${NORMAL}"
echo "${YELLOW}No responsibility for data loss or hardware damage!${NORMAL}"
echo " "
echo "A linux partition large enough to hold the image file"
echo "must be created prior to installation."
echo " "
echo "${YELLOW}This version uses Lilo Boot Loader in the specified partition."
echo "You must therefore make that partition the active/boot one.${NORMAL}"
echo " "
echo "For Grub Boot Loader version, use frugal_grub.sh"
echo " "
echo -n "${CYAN}Enter the target partition to hold DSL image & Lilo ${WHITE}(EXAMPLE: ${YELLOW}hda1): ${NORMAL}" 
read TARGET
if [ -z "$TARGET" ] ; then
  echo "No partition chosen. The script will be terminated."
  echo -n "Press any key. "; read gagme
  exit 1
fi    

DISK="$(echo $TARGET | tr -d [0-9])"

FDISK="$(sfdisk -l /dev/$DISK|grep $TARGET|awk '{print $(NF-1)}'|head -n1)"
if [ "$FDISK" != "83" ]; then
  echo "${RED}Sorry system has not detected a linux partition.${NORMAL}"
  echo "You may need to just reboot, so the system can re-read"
  echo "the partition table, or create your partition(s) with ${YELLOW}cfisk"
  echo "${NORMAL}and then reboot."
  echo "Script is terminating..."
  echo -n "Press any key. "; read gagme
  exit 1
fi

if [ ! -e /mnt/$TARGET ]; then
  mkdir /mnt/$TARGET
fi  

echo "Install from:"
echo "  [L]iveCD or Frugal Install"
echo "  [P]endrive"
echo "  Local [I]so File"
echo "  Fetching latest iso from [W]eb"
echo "  From Poormans via [B]ootfloppy"
echo -n "Choose (l/p/i/w/b): "
read INSTALL
case "$INSTALL" in 
  "L" | "l")
    if [ -f /etc/sysconfig/toram ]; then
       echo "You are running 'toram' mode, not a liveCD!"
       echo -n "Script is terminating... Press any key. "; read gagme
       exit 1
    fi
    SOURCE="/cdrom"
    IMAGE="/cdrom"
    BOOT="/cdrom/boot/isolinux"
    if [ -f "/cdrom/KNOPPIX/boot.img" ]; then  # a syslinux cdrom
       mkdir /mnt/bootimg
       mount -t vfat /cdrom/KNOPPIX/boot.img /mnt/bootimg -o ro,loop
       BOOT="/mnt/bootimg"
    fi   
  ;;   
  "P" | "p")
    if [ -f /etc/sysconfig/toram ]; then
       echo "You are running 'toram' mode!"
       echo -n "Script is terminating... Press Enter key. "; read gagme
       exit 1
    fi
    SOURCE="/cdrom"
    IMAGE="/cdrom"
    BOOT="/cdrom"
  ;;   
  "I" | "i")
    # read source partition from user                                               
    echo -n "${CYAN}Enter the full path to the iso.  ${WHITE}(EXAMPLE: ${YELLOW}/home/dsl/dsl-0.6.0.iso): ${NORMAL}"
    read SOURCE                                                                     
    if [ -z "$SOURCE" ] ; then                                                      
      echo "No partition chosen. The script will be terminated."                    
      echo -n "Press any key. "; read gagme
      exit 1                                                                        
    fi          
  ;;  
  "W" | "w")
    # read ISO from user
    echo "Be sure that your ramdisk has enough space to hold the image file!"
    echo -n "${CYAN}Enter the name of the iso file ${WHITE}(EXAMPLE: ${YELLOW}current.iso): ${NORMAL}"
    read ISO
    if [ -z "$ISO" ] ; then
      ISO="current.iso"
    fi
    URL="$PROTOCOL"://"$MIRROR""current"/"$ISO"
    echo "${GRENN}Standby fetching ${MAGENTA}$URL ${NORMAL}"
    wget "$URL"
    if [ $? != 0 ]; then
      rm -f $ISO
      echo "${RED}Sorry the iso image file could not be retrieved or does not fit"
      echo "on your ramdisk."
      echo "The script is terminating...${NORMAL}"
      echo -n "Press any key. "; read gagme
      exit 1
    fi
    SOURCE="$ISO"
  ;;  
  "B" | "b")
    if [ -f /etc/sysconfig/toram ]; then
       echo "You are running 'toram' mode!"
       echo -n "Script is terminating... Press Enter key. "; read gagme
       exit 1
    fi
    mount /dev/fd0 /mnt/auto/floppy
    if [ "$?" != 0 ]; then
       echo "Error on mounting floppy."
       exit 1
    fi
    SOURCE="/cdrom"
    IMAGE="/cdrom"
    BOOT="/mnt/auto/floppy"
  ;;   
esac  

echo "${CYAN}List boot options, "
echo -n "${WHITE}Example: ${YELLOW}toram ssh nfs syslog lpd monkey ftp${NORMAL}: "
read DAEMONS

echo "${CYAN}Choose language/keyboard if other than english, "
echo -n "${WHITE}Example: ${YELLOW}cs da de es fr nl it pl ru sk${NORMAL}: "
read LANG
if [ ! -z "$LANG" ]; then
   LANG="lang=$LANG"
fi

RESTORE=""; XVESA=""; MYDSL=""
echo -n "${CYAN}Do you wish to specify a default restore partition${NORMAL} (y/..)? "
read answer
if [ "$answer" == "y" ]; then
  echo -n "${CYAN}Enter the partition to be used for backup/restore.${WHITE}(EXAMPLE: ${YELLOW}hda3): ${NORMAL}"
  read DATA
  if [ -z "$DATA" ] ; then
    echo "No partition chosen. The automatic USB/Floppy will be used."
    RESTORE="restore"
  else
    RESTORE="restore=$DATA"  
  fi
else
  echo -n "${CYAN}Use default Xvesa settings, ${YELLOW}Xvesa${NORMAL} (y/..)? "
  read answer
  if [ "$answer" == "y" ]; then
    XVESA="xvesa"
  fi
fi
echo -n "${CYAN}Do you wish to specify a different partition for myDSL${NORMAL} (y/..)? "
read answer
if [ "$answer" == "y" ]; then
  echo -n "${CYAN}Enter the partition to be used for myDSL applications.${WHITE}(EXAMPLE: ${YELLOW}hda3): ${NORMAL}"
  read DATA
  if [ -z "$DATA" ] ; then
    echo "No partition chosen. The automatic /cdrom will be used."
  else
    MYDSL="mydsl=$DATA"  
  fi
fi

echo "${YELLOW}For INSTALL answer y to format, for UPGRADE answer n.${NORMAL} "
echo -n "${RED}Format the target partition ${MAGENTA}/dev/$TARGET${NORMAL} (y/..)? "
read answer
if [ "$answer" == "y" ]; then
  echo
  echo "${RED}Last chance to exit before destroying all data on ${MAGENTA}/dev/$TARGET${NORMAL}"
  echo -n "${CYAN}Continue (y/..)?${NORMAL} "
  read answer
  if [ "$answer" != "y" ]; then
    echo "Aborted.."
    echo -n "Press any key. "; read gagme
    exit 0
  fi
  echo
  echo "${BLUE}Formatting ${MAGENTA}/dev/$TARGET${NORMAL}"
  mke2fs /dev/$TARGET 
fi
mount /mnt/$TARGET  # Mount the new target drive 

echo "${BLUE}Setting up system image on ${MAGENTA}/mnt/$TARGET${NORMAL}"
if [ "$SOURCE" != "/cdrom" ]; then
  # We need to mount the ISO file using the loopback driver.
  # Start by creating a mount point directory; we'll call it "staging"
  [ -d /mnt/staging ] || mkdir /mnt/staging
  mount -t iso9660 -o loop,ro $SOURCE /mnt/staging
  if [ $? != 0 ]; then
    umount /mnt/$TARGET  # unmount the target drive 
    echo "${RED}Sorry $SOURCE not found!"
    echo "Installation not successful!${NORMAL}"
    echo -n "Press any key. "; read gagme
    exit 1
  fi  
  IMAGE="/mnt/staging"
  BOOT="/mnt/staging/boot/isolinux"
fi

#cp -af $IMAGE/* /mnt/$TARGET
mkdir /mnt/$TARGET/KNOPPIX
cp -f $IMAGE/KNOPPIX/KNOPPIX /mnt/$TARGET/KNOPPIX/KNOPPIX

# On the target drive we need a place to put the boot files
mkdir /mnt/$TARGET/boot
cp -f $BOOT/linux24 /mnt/$TARGET/boot/.
cp -f $BOOT/minirt24.gz /mnt/$TARGET/boot/.

sync;sync

CMDLINE="$(cat /proc/cmdline)"
POSITION=$(awk '{print index($0, "vga")}' /proc/cmdline)
if [ "$POSITION" == 0 ]; then
  VGA=791
else
  RESULT=$(awk '{print substr($0,index($0, "vga")+3)}' /proc/cmdline)
  VGA=$(getparam 'vga' "$RESULT")
  if [ -z "$VGA" ]; then
    VGA="$(getbootparam 'vga' 2>/dev/null)"
    if [ -z "$VGA" ]; then
       VGA=791
    fi
  fi
fi    

echo "${BLUE}Setting up lilo${NORMAL}"
cat <<EOF >/mnt/$TARGET/boot/lilo.conf
lba32          #enable large disk support
timeout=30     #wait 3 seconds before auto-booting
vga=$VGA
boot=/dev/$TARGET   #put Lilo boot loader in partition of target disk
root=/dev/$TARGET   #set root to be partition of target disk
image=/boot/linux24
label=DSL
initrd=/boot/minirt24.gz
read-only
APPEND="ramdisk_size=100000 init=/etc/init lang=us apm=power-off hda=scsi hdb=scsi hdc=scsi hdd=scsi hde=scsi hdf=scsi hdg=scsi hdh=scsi nomce nodma quiet frugal BOOT_IMAGE=/KNOPPIX/KNOPPIX fromhd=/dev/$TARGET $RESTORE $DAEMONS $LANG $XVESA $MYDSL"
EOF

chmod 644 /mnt/$TARGET/boot/lilo.conf

ln -sf /mnt/$TARGET/boot /

lilo -C /mnt/$TARGET/boot/lilo.conf

stringinstring "$INSTALL" "Bb" && umount /mnt/auto/floppy
umount /mnt/"$TARGET"
echo "${GREEN}Installation complete.${NORMAL}"
