#!/bin/bash
#(c) 2007 Robert Shingledecker
#Frugal Grub Pendrive Install

. /etc/init.d/dsl-functions

partition(){
/sbin/sfdisk /dev/sda << EOF
0,$SHARE,b
,,83,*
EOF
/bin/mkdosfs /dev/sda1
/sbin/mke2fs /dev/sda2
hdparm -z /dev/sda
echo "${GREEN}Partition and Format Phase Completed.${NORMAL}"
hdparm -z /dev/sda
sleep 3
}

if [ $UID != 0 ]; then exit 1; fi

if [ -f /etc/sysconfig/toram ]; then
   echo "You are not running from LiveCD. Running 'toram' is not supported."
   echo -n "Script will be terminated. Press Enter key."; read gagme
   exit 1
fi
if [ -f /cdrom/KNOPPIX/boot.img ]; then
   echo -n "Not supported from SYSLINUX CD version."; read gagme
   exit 1
fi

clear
echo
echo "${RED}DSL ${WHITE}pendrive GRUB boot USB_HDD ${BLUE}install/update.${NORMAL}"
echo
echo "1. Your BIOS must support USB-HDD booting."
echo "2. Two partitions will be made on device ${WHITE}sda,${NORMAL}" 
echo "      first larger partiton ${WHITE}sda1 ${NORMAL}(vfat) shared, "
echo "      second smaller ${WHITE}sda2 ${NORMAL}(ext2) for DSL/GRUB system."
echo "3. Do not have your pendrive mounted."
echo
echo "${YELLOW}No responsibility for data loss or hardware damage!${NORMAL}"
echo

# Check to help the user avoid accidentally installing to the wrong devicename
echo -n "${CYAN}Verify your USB storage device is ${WHITE}sda ? ${YELLOW}(Y/n): ${NORMAL}"
read ANSWER
if [ "$ANSWER" != "n" ] && [ "$ANSWER" != "N" ] ; then
  SEPARATOR="--------------------------------------------------------"
  SEDGREEN="^[\[1;32m"
  SEDNORMAL="^[\[0;39m"
  dmesg | grep -A 4 "SCSI emulation" | sed "s/--/$SEPARATOR/g;/scsi.* disk/ s/disk/disk$SEDGREEN/g;s/ at /$SEDNORMAL at /g"
fi
ANSWER=

TARGET="sda2"
DISK="$(echo $TARGET | tr -d [0-9])"
DATA="$DISK"1

RESULTS="$(hdparm -g /dev/sda|grep geometry|awk '{print $3}')"
CYCLINDERS=`echo $RESULTS | awk -F/ '{print $1}'`
HEADS=`echo $RESULTS | awk -F/ '{print $2}'`
SECTORS=`echo $RESULTS | awk -F/ '{print $3}'|cut -f1 -d,`
UNITS=`expr $HEADS \* $SECTORS \/ 2`
SYSTEM=`expr 1024 \* 52 \/ $UNITS`
SHARE=`expr $CYCLINDERS - $SYSTEM - 1`
#
echo "${CYAN}List boot options: "
echo -n "${WHITE}Example: ${YELLOW}vga=normal toram ssh tz=US/Pacific lpd monkey ftp${NORMAL}: "
read DAEMONS

echo "${CYAN}Choose language/keyboard if other than english: "
echo -n "${WHITE}Example: ${YELLOW}cs da de es fr nl it pl ru sk${NORMAL}: "
read LANG
if [ ! -z "$LANG" ]; then
   LANG="lang=$LANG"
fi

echo -n "${RED}Last chance. Ready to proceed.${NORMAL} (y/..)? "
read answer
if [ "$answer" != "y" ]; then
  echo "Aborted by user."
  echo "Press Enter key to continue: "; read gagme
  exit 1
fi

echo "${GREEN}Clearing MBR.${NORMAL}"
dd if=/dev/zero of=/dev/sda bs=512 count=1

echo "${GREEN}Partitioning.${NORMAL}"
partition

if [ ! -e /mnt/$TARGET ]; then
  mkdir /mnt/$TARGET
fi  
mount /dev/$TARGET /mnt/$TARGET  # Mount the new target drive 

echo "${GREEN}Setting up GRUB boot loader.${NORMAL}"
if [ ! -d /mnt/$TARGET/boot/grub ]; then
  mkdir -p /mnt/$TARGET/boot/grub
fi
echo '(hd0) /dev/sda' > /mnt/$TARGET/boot/grub/device.map
grub-install --root-directory=/mnt/$TARGET '(hd0)'

echo "default 0" > /mnt/$TARGET/boot/grub/menu.lst
echo "timeout 10" >> /mnt/$TARGET/boot/grub/menu.lst 
echo "title DSL" >> /mnt/$TARGET/boot/grub/menu.lst
echo "root (hd0,1)" >> /mnt/$TARGET/boot/grub/menu.lst
echo "kernel /linux24 ramdisk_size=100000 lang=us quiet vga=791 frugal " $DAEMONS " " $LANG >> /mnt/$TARGET/boot/grub/menu.lst
echo "initrd /minirt24.gz" >> /mnt/$TARGET/boot/grub/menu.lst

echo "${GREEN}Setting up system image on ${MAGENTA}/mnt/$TARGET${NORMAL}"
SOURCE="/cdrom"
IMAGE="/cdrom"
BOOT="/cdrom/boot/isolinux"
if [ -f "/cdrom/KNOPPIX/boot.img" ]; then  # a syslinux cdrom
   mkdir /mnt/bootimg
   mount -t vfat /cdrom/KNOPPIX/boot.img /mnt/bootimg -o ro,loop
   BOOT="/mnt/bootimg"
fi   

if [ ! -d /mnt/$TARGET/KNOPPIX ]; then
  mkdir /mnt/$TARGET/KNOPPIX
fi
cp -f $IMAGE/KNOPPIX/KNOPPIX /mnt/$TARGET/KNOPPIX/KNOPPIX
sync
cp $BOOT/linux24 /mnt/$TARGET/.
sync
cp $BOOT/minirt24.gz /mnt/$TARGET/.
sync
umount /mnt/$TARGET

if [ ! -e /mnt/$DATA ]; then
  mkdir /mnt/$DATA
fi  
mount /dev/$DATA /mnt/$DATA -t vfat # Mount the data partition

mkdir /mnt/$DATA/mydsl
touch /mnt/$DATA/"backup.tar.gz"
umount /mnt/$DATA

echo "${GREEN}Installation Completed.${NORMAL}"
exit 0
